# -*- coding: utf-8 -*-

"""
prismic.utils
~~~~~~~~~~~

This module is mostly for internal use.

"""

import sys


PY3 = sys.version_info[0] == 3

if PY3:
    string_types = str
else:
    string_types = basestring



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (absolute_import, division, print_function, unicode_literals)

from collections import namedtuple, defaultdict
import logging
import cgi
import re
import datetime

log = logging.getLogger(__name__)


class FragmentElement(object):
    pass


class Fragment(object):

    _types = None

    @classmethod
    def from_json(cls, data):
        """Create a corresponding fragment object from json."""

        if not cls._types:
            cls._types = {
                "Image":          Fragment.Image,
                "Color":          Fragment.Color,
                "Text":           Fragment.Text,
                "Select":         Fragment.Text,
                "Number":         Fragment.Number,
                "Date":           Fragment.Date,
                "Timestamp":      Fragment.Timestamp,
                "StructuredText": StructuredText,
                "Link.document":  Fragment.DocumentLink,
                "Link.file":      Fragment.MediaLink,
                "Link.web":       Fragment.WebLink,
                "Embed":          Fragment.Embed,
                "GeoPoint":       Fragment.GeoPoint,
                "Group":          Fragment.Group
            }

        fragment_type = data.get("type")
        f_type = cls._types.get(fragment_type)

        if f_type:
            return f_type(data.get("value"))

        log.warning("fragment_type not found: %s" % fragment_type)

    class WithFragments(object):

        def __init__(self, fragments):
            self.fragments = fragments

        def get(self, field):
            return self.fragments.get(field, None)

        def get_field(self, field):
            return self.fragments.get(field, None)

        def get_all(self, field):
            indexed_key = "^%s(\[\d+\])?$" % field
            return list(v for k, v in list(self.fragments.items()) if re.match(indexed_key, k))

        def get_fragment_type(self, field, f_type):
            fragment = self.fragments.get(field)
            return fragment if isinstance(fragment, f_type) else None

        def get_image(self, field, view="main"):
            fragment = self.get_field(field)
            if isinstance(fragment, Fragment.Image):
                return fragment.get_view(view) if fragment else None
            if view == "main" and isinstance(fragment, StructuredText):
                image = fragment.get_image()
                return image.view if image else None
            return None

        def get_number(self, field):
            return self.get_fragment_type(field, Fragment.Number)

        def get_color(self, field):
            return self.get_fragment_type(field, Fragment.Color)

        def get_text(self, field):
            fragment = self.fragments.get(field)
            if isinstance(fragment, StructuredText):
                texts = [block.text for block in fragment.blocks if isinstance(
                    block, Text)]
                return "\n".join(texts) if texts else None
            elif fragment is None:
                return None
            else:
                return fragment.value

        def get_link(self, field):
            return self.get_fragment_type(field, Fragment.Link)

        def get_embed(self, field):
            return self.get_fragment_type(field, Fragment.Embed)

        def get_date(self, field):
            return self.get_fragment_type(field, Fragment.Date)

        def get_timestamp(self, field):
            return self.get_fragment_type(field, Fragment.Timestamp)

        def get_geopoint(self, field):
            return self.get_fragment_type(field, Fragment.GeoPoint)

        def get_group(self, field):
            return self.get_fragment_type(field, Fragment.Group)

        def get_structured_text(self, field):
            return self.get_fragment_type(field, StructuredText)

        def get_html(self, field, link_resolver):
            """Get the html of a field.

            :param field: String with a name of the field to get.
            :param link_resolver: A resolver function for document links.
            Will be called with :class:`prismic.fragments.Fragment.DocumentLink <prismic.fragments.Fragment.DocumentLink>`
            object as argument. Resolver function should return a string, the local url to the document.
            """
            fragment = self.fragments.get(field)
            return self.fragment_to_html(fragment, link_resolver)

        @staticmethod
        def fragment_to_html(fragment, link_resolver, html_serializer=None):
            if isinstance(fragment, StructuredText):
                return fragment.as_html(link_resolver, html_serializer)
            if isinstance(fragment, Fragment.DocumentLink)\
                    or isinstance(fragment, Fragment.Image)\
                    or isinstance(fragment, Fragment.Image.View):
                return fragment.as_html(link_resolver)
            elif fragment:
                return fragment.as_html
            return None

        def as_html(self, link_resolver):
            html = []
            for key, fragment in list(self.fragments.items()):
                html.append("""<section data-field="%s">""" % key)
                html.append(self.fragment_to_html(fragment, link_resolver))
                html.append("""</section>""")
            return ''.join(html)

    # Links

    class Link(FragmentElement):

        @staticmethod
        def parse(data):
            if data is None:
                return None
            hyperlink_type = data.get("type")
            return {
                "Link.web": Fragment.WebLink,
                "Link.document": Fragment.DocumentLink,
                "Link.image": Fragment.MediaLink,
                "Link.file": Fragment.FileLink
            }.get(hyperlink_type, lambda x: None)(data.get("value"))

    class DocumentLink(Link):
        def __init__(self, value):
            document = value.get("document")

            self.id = document.get("id")
            self.type = document.get("type")
            self.tags = document.get("tags")
            self.slug = document.get("slug")
            self.is_broken = value.get("isBroken")

        def as_html(self, documentlink_resolver, html_serializer=None):
            """Get the DocumentLink as html.

            :param documentlink_resolver: A resolver function will be called with :class:`prismic.fragments.Fragment.DocumentLink <prismic.fragments.Fragment.DocumentLink>` object as argument. Resolver function should return a string, the local url to the document.
            """
            return """<a href="%(link)s">%(slug)s</a>""" % {"link": self.get_url(documentlink_resolver), "slug": self.slug}

        def get_url(self, documentlink_resolver=None):
            if not hasattr(documentlink_resolver, '__call__'):
                raise Exception("documentlink_resolver should be a callable object, but it's: %s" % type(documentlink_resolver))
            return documentlink_resolver(self)

        def get_document_id(self):
            return self.id

        def get_document_type(self):
            return self.type

        def get_document_tags(self):
            return self.tags

        def get_document_slug(self):
            return self.slug

        def __repr__(self):
            return "DocumentLink %s, %s, %s, %s" % (self.id, self.type, self.tags, self.is_broken)

    class WebLink(Link):
        def __init__(self, value):
            self.url = value.get("url")

        @property
        def as_html(self):
            return """<a href="%(url)s">%(url)s</a>""" % self.__dict__

        def get_url(self, link_resolver=None):
            return self.url

    class MediaLink(Link):
        def __init__(self, value):
            self.image = value.get("image")
            self.name = self.image.get("name")
            self.kind = self.image.get("kind")
            self.url = self.image.get("url")
            self.size = self.image.get("size")
            self.height = self.image.get("height")
            self.width = self.image.get("width")

        def as_html(self):
            return "<a href='%(url)s'>%(name)s</a>" % self.__dict__

        def get_url(self, link_resolver=None):
            return self.url

    class FileLink(Link):
        def __init__(self, value):
            self.file = value.get("file")
            self.url = self.file.get("url")
            self.kind = self.file.get("kind")
            self.size = self.file.get("size")
            self.name = self.file.get("name")

        def as_html(self):
            return "<a href='%(url)s'>%(name)s</a>" % self.__dict__

        def get_file(self):
            return self.file

        def get_filename(self):
            return self.name

        def get_url(self, link_resolver=None):
            return self.url

    class Image(FragmentElement):
        _View = namedtuple('View', ['url', 'width', 'height', 'linkTo'])

        class View(FragmentElement):
            """View class"""

            def __init__(self, data):
                self.url = data["url"]
                self.width = data["dimensions"]["width"]
                self.height = data["dimensions"]["height"]
                self.link_to = Fragment.Link.parse(data.get("linkTo"))
                self.label = data.get("label")

            def as_html(self, link_resolver):
                img_tag = """<img src="%(url)s" width="%(width)s" height="%(height)s">""" % {
                    'url': self.url,
                    'width': self.width,
                    'height': self.height
                }
                if self.link_to is None:
                    return img_tag
                else:
                    url = self.link_to.get_url(link_resolver)
                    return """<a href="%(url)s">%(content)s</a>""" % {
                        'url': url,
                        'content': img_tag
                    }

            @property
            def ratio(self):
                return self.width / self.height

        def __init__(self, value):
            main, views, link = value.get("main"), value.get("views"), value.get("linkTo")

            self.main = Fragment.Image.View(main)
            self.views = {
                view_key: Fragment.Image.View(view_value) for (view_key, view_value) in list(views.items())
            }
            self.link_to = Fragment.Link.parse(link)

        def get_view(self, key):
            if key == "main":
                return self.main
            else:
                return self.views.get(key)

        def as_html(self, link_resolver):
            view_html = self.main.as_html(link_resolver)
            if self.link_to is None:
                return view_html
            else:
                return """<a href="%(url)s">%(content)s</a>""" % {
                    'url': self.link_to.get_url(link_resolver),
                    'content': view_html
                }

    class Embed(FragmentElement):
        def __init__(self, value):
            oembed = value.get("oembed")
            self.type = oembed.get("type")
            self.provider = oembed.get("provider_name")
            self.provider = oembed.get("provider_name")
            self.url = oembed.get("embed_url")
            self.width = oembed.get("width")
            self.height = oembed.get("height")
            self.html = oembed.get("html")

        @property
        def as_html(self):
            return ("""<div data-oembed="%(url)s" data-oembed-type="%(type)s" data-oembed-provider="%(provider)s">"""
                    "%(html)s"
                    "</div>") % self.__dict__

    class GeoPoint(FragmentElement):
        def __init__(self, value):
            self.latitude = value.get("latitude")
            self.longitude = value.get("longitude")

        @property
        def as_html(self):
            return ("""<div class="geopoint"><span class="latitude">"""
                    """%(latitude)f</span><span class="longitude">%(longitude)f</span>"""
                    """</div>""") % self.__dict__

    # Basic fragments

    class BasicFragment(FragmentElement):
        def __init__(self, value):
            self.value = value

        def __str__(self):
            return self.value.__str__()

    class Number(BasicFragment):

        @property
        def as_html(self):
            return """<span class="number">%g</span>""" % self.value

    class Color(BasicFragment):

        @property
        def as_html(self):
            return """<span class="color">%s</span>""" % self.value

    class Text(BasicFragment):

        @property
        def as_html(self):
            return """<span class="text">%s</span>""" % cgi.escape(self.value)

    class Date(BasicFragment):

        @property
        def as_datetime(self):
            return datetime.datetime(*map(int, re.split('[^\d]', self.value)))

        @property
        def as_html(self):
            return """<time>%s</time>""" % self.value

    class Timestamp(BasicFragment):

        @property
        def as_datetime(self):
            return datetime.datetime(*map(int, re.split('[^\d]', self.value)))

        @property
        def as_html(self):
            return """<time>%s</time>""" % self.value

    class Group(BasicFragment):

        def __init__(self, value):
            self.value = []
            for elt in value:
                fragments = {}
                for name, frag in elt.items():
                    fragments[name] = Fragment.from_json(frag)
                self.value.append(Fragment.WithFragments(fragments))


class StructuredText(object):

    def __init__(self, values):
        types = {
            "heading1": Block.Heading,
            "heading2": Block.Heading,
            "heading3": Block.Heading,
            "heading4": Block.Heading,
            "paragraph": Block.Paragraph,
            "list-item": Block.ListItem,
            "o-list-item": lambda val: Block.ListItem(val, True),
            "image": lambda val: Block.Image(Fragment.Image.View(val)),
            "embed": lambda val: Block.Embed(Fragment.Embed(val)),
        }

        blocks = []

        for value in values:
            text_type = value.get("type")
            type_class = types.get(text_type)
            if type_class:
                blocks.append(type_class(value))
            else:
                log.warning("StructuredText, type not found: %s" % text_type)

        self.blocks = blocks

    def get_title(self):
        return next(p for p in self.blocks if isinstance(p, Block.Heading))

    def get_first_paragraph(self):
        return next(p for p in self.blocks if isinstance(p, Block.Paragraph))

    def get_image(self):
        return next(p for p in self.blocks if isinstance(p, Block.Image))

    class Group(object):
        def __init__(self, tag, blocks):
            self.tag = tag
            self.blocks = blocks

    def as_html(self, link_resolver, html_serializer=None):
        groups = []
        for block in self.blocks:
            if len(groups) > 0:
                last_one = groups[-1:][0]

                if last_one.tag == "ul" and isinstance(block, Block.ListItem) and not block.is_ordered:
                    last_one.blocks.append(block)
                elif last_one.tag == "ol" and isinstance(block, Block.ListItem) and block.is_ordered:
                    last_one.blocks.append(block)
                elif isinstance(block, Block.ListItem) and not block.is_ordered:
                    groups.append(StructuredText.Group("ul", [block]))
                elif isinstance(block, Block.ListItem) and block.is_ordered:
                    groups.append(StructuredText.Group("ol", [block]))
                else:
                    groups.append(StructuredText.Group(None, [block]))
            else:
                groups.append(StructuredText.Group(None, [block]))

        html = []
        for group in groups:
            if group.tag is not None:
                html.append("<%(tag)s>" % group.__dict__)
            for block in group.blocks:
                content = ""
                if isinstance(block, Text):
                    content = StructuredText.span_as_html(block.text, block.spans, link_resolver, html_serializer)
                html.append(StructuredText.block_as_html(block, content, link_resolver, html_serializer))
            if group.tag is not None:
                html.append("</%(tag)s>" % group.__dict__)

        html_str = ''.join(html)
        log.debug("as_html result: %s" % html_str)
        return html_str

    @staticmethod
    def block_as_html(block, content, link_resolver, html_serializer):
        if html_serializer is not None:
            custom_html = html_serializer(block, content)
            if custom_html is not None:
                return custom_html
        cls = ""
        if isinstance(block, Text) and block.label is not None:
            cls = " class=\"%s\"" % block.label
        if isinstance(block, Block.Heading):
            return "<h%(level)s%(cls)s>%(html)s</h%(level)s>" % {
                "level": block.level,
                "cls": cls,
                "html": content
            }
        elif isinstance(block, Block.Paragraph):
            return "<p%s>%s</p>" % (cls, content)
        elif isinstance(block, Block.ListItem):
            return "<li%s>%s</li>" % (cls, content)
        elif isinstance(block, Block.Image):
            all_classes = ["block-img"]
            if block.view.label is not None:
                all_classes.append(block.view.label)
            return "<p class=\"%s\">%s</p>" % (" ".join(all_classes), block.get_view().as_html(link_resolver))
        elif isinstance(block, Block.Embed):
            return block.get_embed().as_html

    @staticmethod
    def span_write_tag(span, content, link_resolver, html_serializer):
        if html_serializer is not None:
            custom_html = html_serializer(span, content)
            if custom_html is not None:
                return custom_html
        if isinstance(span, Span.Em):
            return "<em>" + content + "</em>"
        elif isinstance(span, Span.Strong):
            return "<strong>" + content + "</strong>"
        elif isinstance(span, Span.Hyperlink):
            return """<a href="%s">""" % span.get_url(link_resolver) + content + "</a>"
        else:
            cls = ""
            if span.label is not None:
                cls = " class=\"%s\"" % span.label
            return """<span%s>%s</span>""" % (cls, content)

    @staticmethod
    def span_as_html(text, spans, link_resolver, html_serializer):
        html = []
        tags_start = defaultdict(list)
        tags_end = defaultdict(list)
        for span in spans:
            tags_start[span.start].append(span)

        for span in reversed(spans):
            tags_end[span.end].append(span)

        index = 0
        stack = []
        for index, letter in enumerate(text):
            if index in tags_end:
                for end_tag in tags_end.get(index):
                    # Close a tag
                    tag = stack.pop()
                    inner_html = StructuredText.span_write_tag(tag["span"], tag["content"], link_resolver, html_serializer)
                    if len(stack) == 0:
                        # The tag was top-level
                        html.append(inner_html)
                    else:
                        # Add the content to the parent tag
                        stack[-1]["content"] += inner_html
            if index in tags_start:

                for span in reversed(sorted(tags_start.get(index), key=lambda s: s.length())):
                    # Open a tag
                    stack.append({
                        "span": span,
                        "content": ""
                    })
            if len(stack) == 0:
                # Top-level text
                html.append(cgi.escape(letter))
            else:
                # Inner text of a span
                stack[-1]["content"] += cgi.escape(letter)

        # Check for the tags after the end of the string
        while len(stack) > 0:
            # Close a tag
            tag = stack.pop()
            inner_html = StructuredText.span_write_tag(tag["span"], tag["content"], link_resolver, html_serializer)
            if len(stack) == 0:
                # The tag was top-level
                html.append(inner_html)
            else:
                # Add the content to the parent tag
                stack[-1]["content"] += inner_html

        return ''.join(html)


class Span(object):

    @classmethod
    def from_json(cls, data):
        return {
            "strong": Span.Strong,
            "em": Span.Em,
            "hyperlink": Span.Hyperlink
        }.get(data.get("type"), Span.SpanElement)(data)

    class SpanElement(object):

        def __init__(self, value):
            self.start = value.get("start")
            self.end = value.get("end")
            if value.get("data") is not None:
                self.label = value.get("data").get("label")
            else:
                self.label = None

        def length(self):
            return self.end - self.start

    class Em(SpanElement):
        pass

    class Strong(SpanElement):
        pass

    class Hyperlink(SpanElement):
        def __init__(self, value):
            super(Span.Hyperlink, self).__init__(value)
            data = value.get('data')
            self.link = Fragment.Link.parse(data)
            if self.link is None:
                log.warning("StructuredText::Span::Hyperlink type not found: %s" % data.get('type'))

        def get_url(self, link_resolver):
            return self.link.get_url(link_resolver)


class Text(object):
    """Base class for blocks"""
    def __init__(self, value):
        self.text = value.get("text")
        self.spans = [Span.from_json(span) for span in value.get("spans")]
        self.label = value.get("label")


class Block(object):

    """A block in a structured text"""
    pass

    class Heading(Text):

        def __init__(self, value):
            super(Block.Heading, self).__init__(value)
            self.level = value.get("type")[-1]

    class Paragraph(Text):

        def __init__(self, value):
            super(Block.Paragraph, self).__init__(value)

    class ListItem(Text):

        def __init__(self, value, is_ordered=False):
            super(Block.ListItem, self).__init__(value)
            self.is_ordered = is_ordered

    class Embed(object):

        def __init__(self, embed):
            self.obj = embed

        def get_embed(self):
            return self.obj

    class Image(object):
        """Block image

        :param view: The Fragment.Image.View object
        """
        def __init__(self, view):
            self.view = view

        def get_view(self):
            return self.view


#!/usr/bin/env python
# -*- coding: utf-8 -*-

fixture_api = """
{
    "refs": [{
        "id": "master",
        "ref": "UgjWQN_mqa8HvPJY",
        "label": "Master",
        "isMasterRef": true
    }, {
        "id": "UkL0hcuvzYUANCrr",
        "ref": "UgjWRd_mqbYHvPJa",
        "label": "San Francisco Grand opening"
    }],
    "bookmarks": {
        "about": "Ue0EDd_mqb8Dhk3j",
        "jobs": "Ue0EHN_mqbwDhk3l",
        "stores": "Ue0EVt_mqd8Dhk3n"
    },
    "types": {
        "blog-post": "Blog post",
        "store": "Store",
        "article": "Site-level article",
        "selection": "Products selection",
        "job-offer": "Job offer",
        "product": "Product"
    },
    "tags": ["Cupcake", "Pie", "Featured", "Macaron"],
    "forms": {
        "everything": {
            "method": "GET",
            "enctype": "application/x-www-form-urlencoded",
            "action": "http://lesbonneschoses.wroom.io/api/documents/search",
            "fields": {
                "ref": {
                    "type": "String"
                },
                "q": {
                    "type": "String"
                }
            }
        },
        "blog":{
           "name":"The Blog",
           "method":"GET",
           "rel":"collection",
           "enctype":"application/x-www-form-urlencoded",
           "action":"http://lesbonneschoses.prismic.io/api/documents/search",
           "fields":{
              "ref":{
                 "type":"String"
              },
              "q":{
                 "default":"[[any(document.type, [\\"blog-post\\"])]]",
                 "type":"String"
              }
           }
        }
    },
    "oauth_initiate": "http://lesbonneschoses.wroom.io/auth",
    "oauth_token": "http://lesbonneschoses.wroom.io/auth/token"
}"""
fixture_search = """[
    {
        "id": "UdUkXt_mqZBObPeS",
        "type": "product",
        "href": "http://lesbonneschoses.wroom.io/api/documents/search?ref=UgjWQN_mqa8HvPJY&q=%5B%5B%3Ad+%3D+at%28document.id%2C+%22UdUkXt_mqZBObPeS%22%29+%5D%5D",
        "tags": [
            "Macaron"
        ],
        "slugs": [
            "vanilla-macaron"
        ],
        "data": {
            "product": {
                "image": {
                    "type": "Image",
                    "value": {
                        "main": {
                            "url": "https://wroomio.s3.amazonaws.com/lesbonneschoses/0417110ebf2dc34a3e8b7b28ee4e06ac82473b70.png",
                            "dimensions": {
                                "width": 500,
                                "height": 500
                            }
                        },
                        "views": {
                            "icon": {
                                "url": "https://wroomio.s3.amazonaws.com/lesbonneschoses/babdc3421037f9af77720d8f5dcf1b84c912c6ba.png",
                                "dimensions": {
                                    "width": 250,
                                    "height": 250
                                }
                            }
                        }
                    }
                },
                "short_lede": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "heading2",
                            "text": "Crispiness and softness, rolled into one",
                            "spans": []
                        }
                    ]
                },
                "description": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "paragraph",
                            "text": "Experience the ultimate vanilla experience. Our vanilla Macarons are made with our very own (in-house) pure extract of Madagascar vanilla, and subtly dusted with our own vanilla sugar (which we make from real vanilla beans).",
                            "spans": [
                                {
                                    "start": 103,
                                    "end": 137,
                                    "type": "strong"
                                },
                                {
                                    "start": 162,
                                    "end": 183,
                                    "type": "strong"
                                }
                            ]
                        }
                    ]
                },
                "testimonial_author": [
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "heading3",
                                "text": "Chef Guillaume Bort",
                                "spans": []
                            }
                        ]
                    }
                ],
                "testimonial_quote": [
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "paragraph",
                                "text": "The taste of pure vanilla is very hard to tame, and therefore, most cooks resort to substitutes. It takes a high-skill chef to know how to get the best of tastes, and Les Bonnes Choses's vanilla macaron does just that. The result is more than a success, it simply is a gastronomic piece of art.",
                                "spans": [
                                    {
                                        "start": 97,
                                        "end": 167,
                                        "type": "strong"
                                    },
                                    {
                                        "start": 167,
                                        "end": 184,
                                        "type": "strong"
                                    },
                                    {
                                        "start": 167,
                                        "end": 184,
                                        "type": "em"
                                    },
                                    {
                                        "start": 184,
                                        "end": 217,
                                        "type": "strong"
                                    }
                                ]
                            }
                        ]
                    }
                ],
                "related": [
                    {
                        "type": "Link.document",
                        "value": {
                            "document": {
                                "id": "UdUjvt_mqVNObPeO",
                                "type": "product",
                                "tags": [
                                    "Macaron"
                                ],
                                "slug": "dark-chocolate-macaron"
                            },
                            "isBroken": false
                        }
                    },
                    {
                        "type": "Link.document",
                        "value": {
                            "document": {
                                "id": "UdUjsN_mqT1ObPeM",
                                "type": "product",
                                "tags": [
                                    "Macaron"
                                ],
                                "slug": "salted-caramel-macaron"
                            },
                            "isBroken": false
                        }
                    }
                ],
                "price": {
                    "type": "Number",
                    "value": 3.55
                },
                "color": {
                    "type": "Color",
                    "value": "#ffeacd"
                },
                "flavour": [
                    {
                        "type": "Select",
                        "value": "Vanilla"
                    }
                ],
                "name": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "heading1",
                            "text": "Vanilla Macaron",
                            "spans": []
                        }
                    ]
                },
                "allergens": {
                    "type": "Text",
                    "value": "Contains almonds, eggs, milk"
                }
            }
        }
    },
    {
        "id": "UdUjsN_mqT1ObPeM",
        "type": "product",
        "href": "http://lesbonneschoses.wroom.io/api/documents/search?ref=UgjWQN_mqa8HvPJY&q=%5B%5B%3Ad+%3D+at%28document.id%2C+%22UdUjsN_mqT1ObPeM%22%29+%5D%5D",
        "tags": [
            "Macaron"
        ],
        "slugs": [
            "salted-caramel-macaron"
        ],
        "data": {
            "product": {
                "image": {
                    "type": "Image",
                    "value": {
                        "main": {
                            "url": "https://wroomio.s3.amazonaws.com/lesbonneschoses/06074de2d9590adddcdb50547108d811af0d9913.png",
                            "dimensions": {
                                "width": 500,
                                "height": 500
                            }
                        },
                        "views": {
                            "icon": {
                                "url": "https://wroomio.s3.amazonaws.com/lesbonneschoses/7accefd1e7204bbca06e8f13b8ef25fdb673ec67.png",
                                "dimensions": {
                                    "width": 250,
                                    "height": 250
                                }
                            }
                        }
                    }
                },
                "short_lede": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "heading2",
                            "text": "Salty-sweety, evermore melty",
                            "spans": []
                        }
                    ]
                },
                "description": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "paragraph",
                            "text": "It's no wonder why our \\"Salted Caramel French Macaron\\" has become our best-selling macaron: on top of the authentic Parisian preparation that has been making our macarons so popular and enjoyed by gourmets, you can also feel the waves of Britanny's ocean as you bite into it. Two of the best French local gastronomies meet in your mouth, and suddenly, it's like French Revolution all over again, contained in a single macaron.",
                            "spans": []
                        }
                    ]
                },
                "testimonial_author": [
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "heading3",
                                "text": "Chef Drobi",
                                "spans": []
                            }
                        ]
                    },
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "heading3",
                                "text": "Chef Guergachi",
                                "spans": []
                            }
                        ]
                    }
                ],
                "testimonial_quote": [
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "paragraph",
                                "text": "You have never known macarons, unless you have tasted Les Bonnes Choses's signature macaron! Salted caramel is very addictive, and even more so when treated with such high standards and respect for the raw material. This is the single best pastry you may taste before long.",
                                "spans": [
                                    {
                                        "start": 54,
                                        "end": 71,
                                        "type": "em"
                                    },
                                    {
                                        "start": 216,
                                        "end": 272,
                                        "type": "strong"
                                    }
                                ]
                            }
                        ]
                    },
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "paragraph",
                                "text": "Try Les Bonnes Choses's salty caramel macaron, and I personally guarantee that you will be grateful to yourself! This is the only dessert I serve in my restaurant which isn't prepared in my own kitchens, and yet, it is always the one I advise first to my most hesitant clients for their dessert. Let's be honest: that way, I just know I'm getting a customer back within weeks...",
                                "spans": [
                                    {
                                        "start": 4,
                                        "end": 21,
                                        "type": "em"
                                    },
                                    {
                                        "start": 51,
                                        "end": 112,
                                        "type": "strong"
                                    }
                                ]
                            }
                        ]
                    }
                ],
                "related": [
                    {
                        "type": "Link.document",
                        "value": {
                            "document": {
                                "id": "UdUjvt_mqVNObPeO",
                                "type": "product",
                                "tags": [
                                    "Macaron"
                                ],
                                "slug": "dark-chocolate-macaron"
                            },
                            "isBroken": false
                        }
                    },
                    {
                        "type": "Link.document",
                        "value": {
                            "document": {
                                "id": "UdUkXt_mqZBObPeS",
                                "type": "product",
                                "tags": [
                                    "Macaron"
                                ],
                                "slug": "vanilla-macaron"
                            },
                            "isBroken": false
                        }
                    },
                    {
                        "type": "Link.document",
                        "value": {
                            "document": {
                                "id": "UdUoc9_mqRlQbPeU",
                                "type": "product",
                                "tags": [
                                    "Macaron"
                                ],
                                "slug": "pistachio-macaron"
                            },
                            "isBroken": false
                        }
                    }
                ],
                "price": {
                    "type": "Number",
                    "value": 3.5
                },
                "color": {
                    "type": "Color",
                    "value": "#db6e09"
                },
                "flavour": [
                    {
                        "type": "Select",
                        "value": "Salted caramel"
                    },
                    {
                        "type": "Select",
                        "value": ""
                    }
                ],
                "name": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "heading1",
                            "text": "Salted Caramel Macaron",
                            "spans": []
                        }
                    ]
                },
                "allergens": {
                    "type": "Text",
                    "value": "Contains almonds, eggs and milk"
                }
            }
        }
    },
    {
        "id": "UebQ4N_mqYEJYF7N",
        "type": "product",
        "href": "http://lesbonneschoses.wroom.io/api/documents/search?ref=UgjWQN_mqa8HvPJY&q=%5B%5B%3Ad+%3D+at%28document.id%2C+%22UebQ4N_mqYEJYF7N%22%29+%5D%5D",
        "tags": [
            "Pie"
        ],
        "slugs": [
            "cherry-lime-pie"
        ],
        "data": {
            "product": {
                "image": {
                    "type": "Image",
                    "value": {
                        "main": {
                            "url": "https://wroomio.s3.amazonaws.com/lesbonneschoses/d88b287f7971c0d5f6d17c90176ac186fcd5ba22.png",
                            "dimensions": {
                                "width": 500,
                                "height": 500
                            }
                        },
                        "views": {
                            "icon": {
                                "url": "https://wroomio.s3.amazonaws.com/lesbonneschoses/4a4acae4e5f3dd466ba2d8c9d2083a411439431d.png",
                                "dimensions": {
                                    "width": 250,
                                    "height": 250
                                }
                            }
                        }
                    }
                },
                "short_lede": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "heading2",
                            "text": "Sweet and sour, but mostly sweet",
                            "spans": []
                        }
                    ]
                },
                "description": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "paragraph",
                            "text": "Cherry flavour in pastries is always a bit aggressive, and usually doesn't care much to let other tastes express themselves. But today, cherry is meeting an opponent who's very able to challenge him: pure Italian-imported lime! As they fight for their right to let you experience their magic the result is an explosion of opposing tastes in your mouth! Chocolate dust will serve as the palate-awakening judge of this heated encounter, while the cherry will attempt to display its domination by sitting on the boxing ring.",
                            "spans": []
                        }
                    ]
                },
                "testimonial_author": [
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "heading3",
                                "text": "Chef Crème",
                                "spans": []
                            }
                        ]
                    },
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "heading3",
                                "text": "Chef Drobi",
                                "spans": []
                            }
                        ]
                    }
                ],
                "testimonial_quote": [
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "paragraph",
                                "text": "If you're looking for one of Les Bonnes Choses's notoriously daring masterpieces... look no more! Whatever your experience with pastry, this will unsettle you, to a point where you'll wonder whether pastry can be taken further than this bold piece of art. To be urgently advised to anyone who believes that pastry can not be an experience by itself.",
                                "spans": [
                                    {
                                        "start": 29,
                                        "end": 46,
                                        "type": "em"
                                    }
                                ]
                            }
                        ]
                    },
                    {
                        "type": "StructuredText",
                        "value": [
                            {
                                "type": "paragraph",
                                "text": "I've had my share of fruit-combination attempts, and lime was always last on my list to combine with berries, because it just seemed wrong. It took a great pastry house like Les Bonnes Choses to suppress my taboo, and realize that what once seems wrong, can later seem insanely right!",
                                "spans": [
                                    {
                                        "start": 174,
                                        "end": 191,
                                        "type": "em"
                                    }
                                ]
                            }
                        ]
                    }
                ],
                "price": {
                    "type": "Number",
                    "value": 3
                },
                "color": {
                    "type": "Color",
                    "value": "#e9221d"
                },
                "flavour": [
                    {
                        "type": "Select",
                        "value": "Lemon/lime"
                    },
                    {
                        "type": "Select",
                        "value": "Berries"
                    }
                ],
                "name": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "heading1",
                            "text": "Cherry-Lime Pie",
                            "spans": []
                        }
                    ]
                },
                "allergens": {
                    "type": "Text",
                    "value": "Contains lime, milk and eggs."
                }
            }
        }
    }
]"""

fixture_structured_lists = """[
    {
        "id": "UinbYMuvzesP4mix",
        "type": "article",
        "href": "http://lesbonneschoses-uinbymuvzesp4mie.prismic.io/api/documents/search?ref=UjnQi8uvzXIAAYEM&q=%5B%5B%3Ad+%3D+at%28document.id%2C+%22UinbYMuvzesP4mix%22%29+%5D%5D",
        "tags": [],
        "slugs": [
            "about-us"
        ],
        "data": {
            "article": {
                "content": {
                    "type": "StructuredText",
                    "value": [
                        {
                            "type": "heading2",
                            "text": "A tale of pastry and passion",
                            "spans": []
                        },
                       {
                            "type": "heading2",
                            "text": "Here we'll test a list",
                            "spans": []
                        },
                        {
                            "type": "paragraph",
                            "text": "Unordered list:",
                            "spans": []
                        },
                        {
                            "type": "list-item",
                            "text": "Element1",
                            "spans": []
                        },
                        {
                            "type": "list-item",
                            "text": "Element2",
                            "spans": []
                        },
                        {
                            "type": "list-item",
                            "text": "Element3",
                            "spans": []
                        },
                        {
                            "type": "paragraph",
                            "text": "Ordered list:",
                            "spans": []
                        },
                        {
                            "type": "o-list-item",
                            "text": "Element1",
                            "spans": []
                        },
                        {
                            "type": "o-list-item",
                            "text": "Element2",
                            "spans": []
                        },
                        {
                            "type": "o-list-item",
                            "text": "Element3",
                            "spans": []
                        }
                    ]
                }
            }
        }
    }
]"""

fixture_empty_paragraph = """{
   "tags":[],
   "data":{
      "announcement":{
         "content":{
               "type":"StructuredText",
               "value":[
                  {
                     "text": "X",
                     "type":"paragraph",
                     "spans":[

                     ]
                  },
                  {
                     "text":"",
                     "type":"paragraph",
                     "spans":[

                     ]
                  },
                  {
                     "text": "Y",
                     "type": "paragraph",
                     "spans": [

                     ]
                  }
               ]
            }
         }
      },
      "id": "123",
      "href": "https://teamup.prismic.io/api/documents/search?ref=aa",
      "type":"announcement",
      "slugs":[]
}"""

fixture_block_labels = """{
   "tags":[],
   "data":{
      "announcement":{
         "content":{
               "type":"StructuredText",
               "value":[
                  {
                     "text": "some code",
                     "type":"paragraph",
                     "label": "code",
                     "spans":[
                     ]
                  }
               ]
            }
         }
      },
      "id": "123",
      "href": "https://teamup.prismic.io/api/documents/search?ref=aa",
      "type":"announcement",
      "slugs":[]
}"""

fixture_store_geopoint = """{
    "id": "UlfoxUnM0wkXYXbq",
    "type": "store",
    "href": "https:\/\/lesbonneschoses-uxlacoaacgazciu.prismic.io\/api\/documents\/search?ref=U_yGzjAAAC8AsL1a&q=%5B%5B%3Ad+%3D+at%28document.id%2C+%22UlfoxUnM0wkXYXbq%22%29+%5D%5D",
    "tags": [

    ],
    "slugs": [
        "san-francisco-pier-39",
        "san-francisco"
    ],
    "linked_documents": [

    ],
    "data": {
        "store": {
            "name": {
                "type": "StructuredText",
                "value": [
                    {
                        "type": "heading1",
                        "text": "San Francisco Pier 39",
                        "spans": [

                        ]
                    }
                ]
            },
            "address": {
                "type": "Text",
                "value": "625 Market Street"
            },
            "city": {
                "type": "Text",
                "value": "San Francisco, CA"
            },
            "zipcode": {
                "type": "Text",
                "value": "94105"
            },
            "country": {
                "type": "Select",
                "value": "United States"
            },
            "coordinates": {
                "type": "GeoPoint",
                "value": {
                    "latitude": 37.777431229812,
                    "longitude": -122.41541862488
                }
            },
            "description": {
                "type": "StructuredText",
                "value": [
                    {
                        "type": "paragraph",
                        "text": "A haven of delicacies in the midst of the City by the Bay.",
                        "spans": [

                        ]
                    }
                ]
            },
            "image": {
                "type": "Image",
                "value": {
                    "main": {
                        "url": "https:\/\/prismic-io.s3.amazonaws.com\/lesbonneschoses-uxlacoaacgazciu\/eb85bc78fdf0f18a0fd3b7a3fc829768e66ea4f0.jpg",
                        "alt": "",
                        "copyright": "",
                        "dimensions": {
                            "width": 1500,
                            "height": 500
                        }
                    },
                    "views": {
                        "medium": {
                            "url": "https:\/\/prismic-io.s3.amazonaws.com\/lesbonneschoses-uxlacoaacgazciu\/7b8d054ea5428009ee6f2cdace249dc89c68199d.jpg",
                            "alt": "",
                            "copyright": "",
                            "dimensions": {
                                "width": 800,
                                "height": 250
                            }
                        },
                        "icon": {
                            "url": "https:\/\/prismic-io.s3.amazonaws.com\/lesbonneschoses-uxlacoaacgazciu\/0ca27446c40d08cb0d732d4b9726e2001fa7d552.jpg",
                            "alt": "",
                            "copyright": "",
                            "dimensions": {
                                "width": 250,
                                "height": 250
                            }
                        }
                    }
                }
            },
            "monday": [
                {
                    "type": "Text",
                    "value": "9am - 10pm"
                }
            ],
            "tuesday": [
                {
                    "type": "Text",
                    "value": "9am - 10pm"
                }
            ],
            "wednesday": [
                {
                    "type": "Text",
                    "value": "9am - 10pm"
                }
            ],
            "thursday": [
                {
                    "type": "Text",
                    "value": "9am - 10pm"
                }
            ],
            "friday": [
                {
                    "type": "Text",
                    "value": "9am - 10pm"
                }
            ],
            "saturday": [
                {
                    "type": "Text",
                    "value": "9am - 10pm"
                }
            ],
            "sunday": [
                {
                    "type": "Text",
                    "value": "9am - 10pm"
                }
            ]
        }
    }
}"""

fixture_groups = """{
    "id": "UrOaNwEAAM2OpbPy",
    "type": "contributor",
    "href": "http://micro.prismic.io/api/documents/search?ref=U9uFvTQAADAAfFaO&q=%5B%5B%3Ad+%3D+at%28document.id%2C+%22UrOaNwEAAM2OpbPy%22%29+%5D%5D",
    "tags": [],
    "slugs": ["peter-wong"],
    "linked_documents": [],
    "data": {
        "contributor": {
            "links": {
                "type": "Group",
                "value": [
                    {
                        "link": {
                            "type": "Link.web",
                            "value": {
                                "url": "https://twitter.com/peetwong"
                            }
                        },
                        "label": {
                            "type": "Text",
                            "value": "Twitter account"
                        }
                    },
                    {
                        "link": {
                            "type": "Link.web",
                            "value": {
                                "url": "https://worldchanger.prismic.io/api/documents/search?ref=UqEXggEAAOWRr1v_&q=%5B%5B%3Ad%20%3D%20at(document.id%2C%20%22UpUukAEAAIRb1wu7%22)%5D%5D"
                            }
                        },
                        "label": {
                            "type": "Text",
                            "value": "URI on WorldChanger's prismic.io repository"
                        }
                    }
                ]
            }
        }
    }
}"""

fixture_image_links = """{
    "type": "StructuredText",
    "value": [{
        "spans": [],
        "text": "Here is some introductory text.",
        "type": "paragraph"
    }, {
        "spans": [],
        "text": "The following image is linked.",
        "type": "paragraph"
    }, {
        "alt": "",
        "copyright": "",
        "dimensions": {
            "height": 129,
            "width": 260
        },
        "linkTo": {
            "type": "Link.web",
            "value": {
                "url": "http://google.com/"
            }
        },
        "type": "image",
        "url": "http://fpoimg.com/129x260"
    }, {
        "spans": [{
            "end": 20,
            "start": 0,
            "type": "strong"
        }],
        "text": "More important stuff",
        "type": "paragraph"
    }, {
        "spans": [],
        "text": "The next is linked to a valid document:",
        "type": "paragraph"
    }, {
        "alt": "",
        "copyright": "",
        "dimensions": {
            "height": 400,
            "width": 400
        },
        "linkTo": {
            "type": "Link.document",
            "value": {
                "document": {
                    "id": "UxCQFFFFFFFaaYAH",
                    "slug": "something-fantastic",
                    "type": "lovely-thing"
                },
                "isBroken": false
            }
        },
        "type": "image",
        "url": "http://fpoimg.com/400x400"
    }, {
        "spans": [],
        "text": "The next is linked to a broken document:",
        "type": "paragraph"
    }, {
        "alt": "",
        "copyright": "",
        "dimensions": {
            "height": 250,
            "width": 250
        },
        "linkTo": {
            "type": "Link.document",
            "value": {
                "document": {
                    "id": "UxERPAEAAHQcsBUH",
                    "slug": "-",
                    "type": "event-calendar"
                },
                "isBroken": true
            }
        },
        "type": "image",
        "url": "http://fpoimg.com/250x250"
    }, {
        "spans": [],
        "text": "One more image, this one is not linked:",
        "type": "paragraph"
    }, {
        "alt": "",
        "copyright": "",
        "dimensions": {
            "height": 199,
            "width": 300
        },
        "type": "image",
        "url": "http://fpoimg.com/199x300"
    }]
}"""

fixture_custom_html = """{
    "type": "StructuredText",
    "value": [{
        "spans": [],
        "text": "Here is some introductory text.",
        "type": "paragraph"
    }, {
        "spans": [],
        "text": "The following image is linked.",
        "type": "paragraph"
    }, {
        "alt": "",
        "copyright": "",
        "dimensions": {
            "height": 129,
            "width": 260
        },
        "linkTo": {
            "type": "Link.web",
            "value": {
                "url": "http://google.com/"
            }
        },
        "type": "image",
        "url": "http://fpoimg.com/129x260"
    }, {
        "spans": [{
            "end": 20,
            "start": 0,
            "type": "strong"
        }],
        "text": "More important stuff",
        "type": "paragraph"
    }, {
        "spans": [],
        "text": "The next is linked to a valid document:",
        "type": "paragraph"
    }, {
        "alt": "",
        "copyright": "",
        "dimensions": {
            "height": 400,
            "width": 400
        },
        "linkTo": {
            "type": "Link.document",
            "value": {
                "document": {
                    "id": "UxCQFFFFFFFaaYAH",
                    "slug": "something-fantastic",
                    "type": "lovely-thing"
                },
                "isBroken": false
            }
        },
        "type": "image",
        "url": "http://fpoimg.com/400x400"
    }, {
        "spans": [],
        "text": "The next is linked to a broken document:",
        "type": "paragraph"
    }, {
        "alt": "",
        "copyright": "",
        "dimensions": {
            "height": 250,
            "width": 250
        },
        "linkTo": {
            "type": "Link.document",
            "value": {
                "document": {
                    "id": "UxERPAEAAHQcsBUH",
                    "slug": "-",
                    "type": "event-calendar"
                },
                "isBroken": true
            }
        },
        "type": "image",
        "url": "http://fpoimg.com/250x250"
    }, {
        "spans": [],
        "text": "One more image, this one is not linked:",
        "type": "paragraph"
    }, {
        "alt": "",
        "copyright": "",
        "dimensions": {
            "height": 199,
            "width": 300
        },
        "type": "image",
        "url": "http://fpoimg.com/199x300"
    }, {
        "type": "paragraph",
        "text": "This paragraph contains an hyperlink.",
        "spans": [{
            "start": 5,
            "end": 14,
            "type": "hyperlink",
            "data": {
                "type": "Link.document",
                "value": {
                    "document": {
                        "id": "UlfoxUnM0wkXYXbu",
                        "type": "blog-post",
                        "tags": [],
                        "slug": "les-bonnes-chosess-internship-a-testimony"
                    },
                    "isBroken": false
                }
            }
        }]
    }]
}"""

fixture_spans_labels = """{
    "type": "StructuredText",
    "value": [{
        "spans": [{
            "type": "em",
            "start": 4,
            "end": 9
        }, {
            "type": "strong",
            "start": 4,
            "end": 14
        }],
        "text": "Two spans with the same start",
        "type": "paragraph"
    }, {
        "spans": [{
            "type": "em",
            "start": 4,
            "end": 14
        }, {
            "type": "strong",
            "start": 4,
            "end": 9
        }],
        "text": "Two spans with the same start",
        "type": "paragraph"
    }, {
         "spans": [{
            "type": "label",
            "start": 14,
            "end": 17,
            "data": {
                "label": "tip"
            }
        }],
        "text": "Span till the end",
        "type": "paragraph"
    }]
}"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for Prismic library"""

from __future__ import (absolute_import, division, print_function, unicode_literals)

from prismic.cache import ShelveCache
from prismic.exceptions import InvalidTokenError, AuthorizationNeededError, InvalidURLError
from .test_prismic_fixtures import fixture_api, fixture_search, fixture_groups, \
    fixture_structured_lists, fixture_empty_paragraph, fixture_store_geopoint, \
    fixture_image_links, fixture_spans_labels, fixture_block_labels, fixture_custom_html
import time
import json
import logging
import prismic
from prismic import predicates
import unittest

# logging.basicConfig(level=logging.DEBUG)
# log = logging.getLogger(__name__)


class PrismicTestCase(unittest.TestCase):
    def setUp(self):
        """Init the api url and the token identifier."""
        self.api_url = "http://lesbonneschoses.prismic.io/api"
        self.token = "MC5VZ2phOGRfbXFaOEl2UEpj.dO-_ve-_ve-_ve-_vSFRBzXvv71V77-977-9BO-_vVbvv71k77-9Cu-_ve-_vQTvv71177-9eQpcUE3vv70"
        self.fixture_api = json.loads(fixture_api)
        self.fixture_search = json.loads(fixture_search)
        self.fixture_structured_lists = json.loads(fixture_structured_lists)
        self.fixture_empty_paragraph = json.loads(fixture_empty_paragraph)
        self.fixture_block_labels = json.loads(fixture_block_labels)
        self.fixture_store_geopoint = json.loads(fixture_store_geopoint)
        self.fixture_groups = json.loads(fixture_groups)
        self.fixture_image_links = json.loads(fixture_image_links)
        self.fixture_spans_labels = json.loads(fixture_spans_labels)
        self.fixture_custom_html = json.loads(fixture_custom_html)

        self.api = prismic.Api(self.fixture_api, self.token, ShelveCache("prismictest"))

    def tearDown(self):
        """Teardown."""

    @staticmethod
    def link_resolver(document_link):
        if document_link.is_broken:
            return "#broken"
        else:
            return "/document/%s/%s" % (document_link.id, document_link.slug)

    @staticmethod
    def html_serializer(element, content):
        if isinstance(element, prismic.fragments.Block.Image):
            return element.get_view().as_html(PrismicTestCase.link_resolver)
        if isinstance(element, prismic.fragments.Span.Hyperlink):
            return """<a class="some-link" href="%s">""" % element.get_url(PrismicTestCase.link_resolver) + content + "</a>"
        return None


class ApiIntegrationTestCase(PrismicTestCase):
    """Doing real HTTP requests to test API data fetching"""

    def setUp(self):
        super(ApiIntegrationTestCase, self).setUp()
        self.api = prismic.get(self.api_url, self.token)

    def test_get_api(self):
        self.assertTrue(len(self.api.forms) > 0)

    def test_api_get_errors(self):
        with self.assertRaises(InvalidTokenError):
            prismic.get(self.api_url, "wrong")

        with self.assertRaises(AuthorizationNeededError):
            prismic.get(self.api_url, "")

        with self.assertRaises(InvalidURLError):
            prismic.get("htt://wrong_on_purpose", "")

    def test_search_form(self):
        blog = self.api.form("blog")
        blog.ref(self.api.get_master())
        docs = blog.submit().documents
        self.assertEqual(len(docs), 6)
        self.assertEqual(docs[0].type, "blog-post")

    def test_search_form_orderings(self):
        blog = self.api.form("blog")
        blog.ref(self.api.get_master())
        blog.orderings("""[my.blog-post.date]""")
        docs = blog.submit().documents
        self.assertEqual(docs[0].slug, 'les-bonnes-chosess-internship-a-testimony')
        self.assertEqual(docs[1].slug, 'get-the-right-approach-to-ganache')

    def test_search_form_page_size(self):
        blog = self.api.form("blog").page_size(2)
        blog.ref(self.api.get_master())
        response = blog.submit()
        self.assertEqual(len(response.documents), 2)
        self.assertEqual(response.results_per_page, 2)

    def test_search_form_first_page(self):
        blog = self.api.form("blog").pageSize(4)
        blog.ref(self.api.get_master())
        response = blog.submit()
        self.assertEqual(response.page, 1)
        self.assertEqual(len(response.documents), 4)
        self.assertEqual(response.results_size, len(response.documents))
        self.assertIsNone(response.prev_page)
        self.assertEqual(response.next_page,
                         'http://lesbonneschoses.prismic.io/api/documents/search?ref=UlfoxUnM08QWYXdl&q=%5B%5B%3Ad+%3D+any%28document.type%2C+%5B%22blog-post%22%5D%29%5D%5D&page=2&pageSize=4')

    def test_search_form_last_page(self):
        blog = self.api.form("blog").pageSize(4).page(2)
        blog.ref(self.api.get_master())
        response = blog.submit()
        self.assertEqual(response.page, 2)
        self.assertEqual(len(response.documents), 2)
        self.assertEqual(response.results_size, len(response.documents))
        self.assertIsNone(response.next_page)
        self.assertEqual(response.prev_page,
                         'http://lesbonneschoses.prismic.io/api/documents/search?ref=UlfoxUnM08QWYXdl&q=%5B%5B%3Ad+%3D+any%28document.type%2C+%5B%22blog-post%22%5D%29%5D%5D&page=1&pageSize=4')

    def test_search_form_count(self):
        blog = self.api.form("blog")
        blog.ref(self.api.get_master())
        nb_docs = blog.count()
        self.assertEqual(nb_docs, 6)

    def test_linked_documents(self):
        gift_box = self.api\
            .form("everything")\
            .ref(self.api.get_master())\
            .query("[[:d = at(document.id, \"UlfoxUnM0wkXYXbZ\")]]")\
            .submit()\
            .documents[0]
        linked = gift_box.linked_documents
        self.assertEqual(len(linked), 3)


class ApiTestCase(PrismicTestCase):
    def test_get_ref(self):
        self.assertTrue(self.api.get_ref("Master").ref == "UgjWQN_mqa8HvPJY")

    def test_master(self):
        self.assertTrue(self.api.get_master().ref == "UgjWQN_mqa8HvPJY")
        self.assertTrue(self.api.get_master().id == "master")


class TestSearchFormTestCase(PrismicTestCase):
    def test_document(self):
        docs = [prismic.Document(doc) for doc in self.fixture_search]
        self.assertTrue(len(docs) == 3)
        doc = docs[0]
        self.assertTrue(doc.slug == "vanilla-macaron")

    def test_empty_slug(self):
        doc_json = self.fixture_search[0]
        doc_json["slugs"] = None
        doc = prismic.Document(doc_json)
        self.assertTrue(doc.slug == "-")

    def test_as_html(self):
        doc_json = self.fixture_search[0]
        doc = prismic.Document(doc_json)
        expected_html = """<section data-field="product.allergens"><span class="text">Contains almonds, eggs, milk</span></section><section data-field="product.image"><img src="https://wroomio.s3.amazonaws.com/lesbonneschoses/0417110ebf2dc34a3e8b7b28ee4e06ac82473b70.png" width="500" height="500"></section><section data-field="product.short_lede"><h2>Crispiness and softness, rolled into one</h2></section><section data-field="product.testimonial_author[0]"><h3>Chef Guillaume Bort</h3></section><section data-field="product.related[0]"><a href="document/UdUjvt_mqVNObPeO">dark-chocolate-macaron</a></section><section data-field="product.name"><h1>Vanilla Macaron</h1></section><section data-field="product.related[1]"><a href="document/UdUjsN_mqT1ObPeM">salted-caramel-macaron</a></section><section data-field="product.testimonial_quote[0]"><p>The taste of pure vanilla is very hard to tame, and therefore, most cooks resort to substitutes. <strong>It takes a high-skill chef to know how to get the best of tastes, and <strong><em></strong>Les Bonnes Choses<strong></em></strong>'s vanilla macaron does just that</strong>. The result is more than a success, it simply is a gastronomic piece of art.</p></section><section data-field="product.flavour[0]"><span class="text">Vanilla</span></section><section data-field="product.price"><span class="number">3.55</span></section><section data-field="product.color"><span class="color">#ffeacd</span></section><section data-field="product.description"><p>Experience the ultimate vanilla experience. Our vanilla Macarons are made with our very own (in-house) <strong>pure extract of Madagascar vanilla</strong>, and subtly dusted with <strong>our own vanilla sugar</strong> (which we make from real vanilla beans).</p></section>"""
        doc_html = doc.as_html(lambda link_doc: "document/%s" % link_doc.id)
        # Comparing len rather than actual strings because json loading is not in a deterministic order for now
        self.assertEqual(len(expected_html), len(doc_html))

    def test_default_params(self):
        blog = self.api.form("blog")
        self.assertEqual(len(blog.data), 1)
        self.assertEqual(blog.data["q"], ["[[any(document.type, [\"blog-post\"])]]"])

    def test_query_append_value(self):
        blog = self.api.form("blog")
        blog.query("[[bar]]")
        self.assertEqual(len(blog.data), 1)
        self.assertEqual(blog.data["q"], ["[[any(document.type, [\"blog-post\"])]]", "[[bar]]"])

    def test_ref_replace_value(self):
        blog = self.api.form("blog")
        blog.ref("foo")
        self.assertEqual(len(blog.data), 2)
        self.assertEqual(blog.data["ref"], "foo")
        blog.ref("bar")
        self.assertEqual(len(blog.data), 2)
        self.assertEqual(blog.data["ref"], "bar")

    def test_set_page_size(self):
        blog = self.api.form("blog")
        blog.page_size(3)
        self.assertEqual(len(blog.data), 2)
        self.assertEqual(blog.data["pageSize"], 3)

    def test_set_page(self):
        blog = self.api.form("blog")
        blog.page(3)
        self.assertEqual(len(blog.data), 2)
        self.assertEqual(blog.data["page"], 3)


class TestFragmentsTestCase(PrismicTestCase):
    def setUp(self):
        super(TestFragmentsTestCase, self).setUp()
        doc_json = self.fixture_search[0]
        self.doc = prismic.Document(doc_json)

    def test_image(self):
        doc = self.doc
        self.assertEqual(doc.get_image("product.image", "main").width, 500)
        self.assertEqual(doc.get_image("product.image", "icon").width, 250)
        expected_html = \
            ("""<img """
             """src="https://wroomio.s3.amazonaws.com/lesbonneschoses/babdc3421037f9af77720d8f5dcf1b84c912c6ba.png" """
             """width="250" height="250">""")
        self.assertEqual(expected_html, doc.get_image("product.image", "icon").as_html(PrismicTestCase.link_resolver))

    def test_number(self):
        doc = self.doc
        self.assertEqual(doc.get_number("product.price").__str__(), "3.55")

    def test_color(self):
        doc = self.doc
        self.assertEqual(doc.get_color("product.color").__str__(), "#ffeacd")

    def test_text(self):
        doc = self.doc
        self.assertEqual(doc.get_text("product.allergens").__str__(), "Contains almonds, eggs, milk")

        text = prismic.Fragment.Text("a&b 42 > 41")
        self.assertEqual(text.as_html, '<span class="text">a&amp;b 42 &gt; 41</span>', "HTML escape")

    def test_structured_text_heading(self):
        doc = self.doc
        html = doc.get_html("product.short_lede", lambda x: "/x")
        self.assertEqual("<h2>Crispiness and softness, rolled into one</h2>", html)

    def test_structured_text_paragraph(self):
        span_sample_data = {"type": "paragraph",
                            "text": "To be or not to be ?",
                            "spans": [
                                {"start": 3, "end": 5, "type": "strong"},
                                {"start": 16, "end": 18, "type": "strong"},
                                {"start": 3, "end": 5, "type": "em"}
                            ]}
        p = prismic.fragments.StructuredText([span_sample_data])
        p_html = p.as_html(lambda x: "/x")
        self.assertEqual(p_html, "<p>To <em><strong>be</strong></em> or not to <strong>be</strong> ?</p>")

        p = prismic.fragments.StructuredText([{"type": "paragraph", "text": "a&b 42 > 41", "spans": []}])
        p_html = p.as_html(lambda x: "/x")
        self.assertEqual(p_html, "<p>a&amp;b 42 &gt; 41</p>", "Paragraph HTML escape")

        p = prismic.fragments.StructuredText([{"type": "heading2", "text": "a&b 42 > 41", "spans": []}])
        p_html = p.as_html(lambda x: "/x")
        self.assertEqual(p_html, "<h2>a&amp;b 42 &gt; 41</h2>", "Header HTML escape")

    def test_spans(self):
        doc_json = self.fixture_spans_labels
        p = prismic.fragments.StructuredText(doc_json.get("value"))
        p_html = p.as_html(lambda x: "/x")
        self.assertEqual(p_html, ("""<p>Two <strong><em>spans</em> with</strong> the same start</p>"""
                                  """<p>Two <em><strong>spans</strong> with</em> the same start</p>"""
                                  """<p>Span till the <span class="tip">end</span></p>"""))

    def test_lists(self):
        doc_json = self.fixture_structured_lists[0]
        doc = prismic.Document(doc_json)
        doc_html = doc.get_structured_text("article.content").as_html(lambda x: "/x")
        expected = ("""<h2>A tale of pastry and passion</h2>"""
                    """<h2>Here we'll test a list</h2>"""
                    """<p>Unordered list:</p>"""
                    """<ul><li>Element1</li><li>Element2</li><li>Element3</li></ul>"""
                    """<p>Ordered list:</p><ol><li>Element1</li><li>Element2</li><li>Element3</li></ol>""")
        self.assertEqual(doc_html, expected)

    def test_empty_paragraph(self):
        doc_json = self.fixture_empty_paragraph
        doc = prismic.Document(doc_json)

        doc_html = doc.get_field('announcement.content').as_html(PrismicTestCase.link_resolver)
        expected = """<p>X</p><p></p><p>Y</p>"""
        self.assertEqual(doc_html, expected)

    def test_block_labels(self):
        doc_json = self.fixture_block_labels
        doc = prismic.Document(doc_json)

        doc_html = doc.get_field('announcement.content').as_html(PrismicTestCase.link_resolver)
        expected = """<p class="code">some code</p>"""
        self.assertEqual(doc_html, expected)

    def test_get_text(self):
        doc_json = self.fixture_search[0]
        doc = prismic.Document(doc_json)
        self.assertEqual(doc.get_text('product.description'), 'Experience the ultimate vanilla experience. Our vanilla Macarons are made with our very own (in-house) pure extract of Madagascar vanilla, and subtly dusted with our own vanilla sugar (which we make from real vanilla beans).')

    def test_document_link(self):
        test_paragraph = {
            "type": "paragraph",
            "text": "bye",
            "spans": [
                {
                    "start": 0,
                    "end": 3,
                    "type": "hyperlink",
                    "data": {
                        "type": "Link.document",
                        "value": {
                            "document": {
                                "id": "UbiYbN_mqXkBOgE2", "type": "article", "tags": ["blog"], "slug": "-"
                            },
                            "isBroken": False
                        }
                    }
                },
                {"start": 0, "end": 3, "type": "strong"}
            ]
        }
        p = prismic.fragments.StructuredText([test_paragraph])

        def link_resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)

        p_html = p.as_html(link_resolver)
        self.assertEqual(p_html, """<p><strong><a href="/document/UbiYbN_mqXkBOgE2/-">bye</a></strong></p>""")

    def test_geo_point(self):
        store = prismic.Document(self.fixture_store_geopoint)
        geopoint = store.get_field("store.coordinates")
        self.assertEqual(geopoint.as_html,
                         ("""<div class="geopoint"><span class="latitude">37.777431</span>"""
                          """<span class="longitude">-122.415419</span></div>"""))

    def test_group(self):
        contributor = prismic.Document(self.fixture_groups)
        links = contributor.get_group("contributor.links")
        self.assertEquals(len(links.value), 2)

    def test_image_links(self):
        self.maxDiff = 10000
        text = prismic.fragments.StructuredText(self.fixture_image_links.get('value'))

        self.assertEqual(
            text.as_html(PrismicTestCase.link_resolver),
            ('<p>Here is some introductory text.</p>'
             '<p>The following image is linked.</p>'
             '<p class=\"block-img\"><a href=\"http://google.com/\">'
             '<img src=\"http://fpoimg.com/129x260\" width=\"260\" height=\"129\"></a></p>'
             '<p><strong>More important stuff</strong></p><p>The next is linked to a valid document:</p>'
             '<p class=\"block-img\"><a href=\"/document/UxCQFFFFFFFaaYAH/something-fantastic\">'
             '<img src=\"http://fpoimg.com/400x400\" width=\"400\" height=\"400\"></a></p>'
             '<p>The next is linked to a broken document:</p><p class=\"block-img\"><a href=\"#broken\">'
             '<img src=\"http://fpoimg.com/250x250\" width=\"250\" height=\"250\"></a></p>'
             '<p>One more image, this one is not linked:</p><p class=\"block-img\">'
             '<img src=\"http://fpoimg.com/199x300\" width=\"300\" height=\"199\"></p>'))

    def test_custom_html(self):
        self.maxDiff = 10000
        text = prismic.fragments.StructuredText(self.fixture_custom_html.get('value'))

        self.assertEqual(
            text.as_html(PrismicTestCase.link_resolver, PrismicTestCase.html_serializer),
            ('<p>Here is some introductory text.</p>'
             '<p>The following image is linked.</p>'
             '<a href=\"http://google.com/\"><img src=\"http://fpoimg.com/129x260\" width=\"260\" height=\"129\"></a>'
             '<p><strong>More important stuff</strong></p><p>The next is linked to a valid document:</p>'
             '<a href=\"/document/UxCQFFFFFFFaaYAH/something-fantastic\">'
             '<img src=\"http://fpoimg.com/400x400\" width=\"400\" height=\"400\"></a>'
             '<p>The next is linked to a broken document:</p><a href=\"#broken\">'
             '<img src=\"http://fpoimg.com/250x250\" width=\"250\" height=\"250\"></a>'
             '<p>One more image, this one is not linked:</p>'
             '<img src=\"http://fpoimg.com/199x300\" width=\"300\" height=\"199\">'
             '<p>This <a class="some-link" href="/document/UlfoxUnM0wkXYXbu/les-bonnes-chosess-internship-a-testimony">'
             'paragraph</a> contains an hyperlink.</p>'))


class PredicatesTestCase(PrismicTestCase):

    def test_at(self):
        f = self.api\
            .form("everything")\
            .ref(self.api.get_master())\
            .query(predicates.at('document.id', 'UlfoxUnM0wkXYXbZ'))
        self.assertEqual(f.data['q'], ["[[:d = at(document.id, \"UlfoxUnM0wkXYXbZ\")]]"])

    def test_any(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.any('document.type', ['article', 'blog-post']))
        self.assertEqual(f.data['q'], ['[[:d = any(document.type, ["article", "blog-post"])]]'])

    def test_similar(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.similar('idOfSomeDocument', 10))
        self.assertEqual(f.data['q'], ['[[:d = similar("idOfSomeDocument", 10)]]'])

    def test_multiple_predicates(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(
                predicates.month_after('my.blog-post.publication-date', 4),
                predicates.month_before('my.blog-post.publication-date', 'December')
            )
        self.assertEqual(f.data['q'], ['[[:d = date.month-after(my.blog-post.publication-date, 4)][:d = date.month-before(my.blog-post.publication-date, "December")]]'])

    def test_number_lt(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.lt('my.blog-post.publication-date', 4))
        self.assertEqual(f.data['q'], ['[[:d = number.lt(my.blog-post.publication-date, 4)]]'])

    def test_number_in_range(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.in_range('my.product.price', 2, 4.5))
        self.assertEqual(f.data['q'], ['[[:d = number.inRange(my.product.price, 2, 4.5)]]'])

    def test_geopoint_near(self):
        f = self.api \
            .form("everything") \
            .ref(self.api.get_master()) \
            .query(predicates.near('my.store.coordinates', 40.689757, -74.0451453, 15))
        self.assertEqual(f.data['q'], ['[[:d = geopoint.near(my.store.coordinates, 40.689757, -74.0451453, 15)]]'])


class TestCache(unittest.TestCase):

    def setUp(self):
        self.cache = ShelveCache("cachetest")

    def test_set_get(self):
        self.cache.set("foo", "bar", 3600)
        self.assertEqual(self.cache.get("foo"), "bar")

    def test_expiration(self):
        self.cache.set("toto", "tata", 2)
        time.sleep(3)
        self.assertIsNone(self.cache.get("toto"))


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-


class Experiments(object):
    """
    Holds informations about current experiments

    :ivar array<Experiment> running: list of currently running experiments
    :ivar array<Experiment> draft: list of drafts
    """

    def __init__(self, draft, running):
        self.draft = draft
        self.running = running

    def ref_from_cookie(self, cookie):
        """
        From a cookie set by the Prismic.io toolbar, identify the corresponding reference.

        :param cookie: the cookie set by the Prismic.io toolbar and received from the request.
        :return: the reference. May be None if it can't be found.
        """
        if cookie is None:
            return None
        splitted = cookie.strip().split("%20")
        experiment = None
        if len(splitted) >= 2:
            experiment = next((exp for exp in self.running if exp.google_id == splitted[0]), None)
        if experiment is None:
            return None
        var_index = int(splitted[1])
        if -1 < var_index < len(experiment.variations):
            return experiment.variations[var_index].ref

    def current(self):
        """
        :return: the currently running experiment, or None
        """
        try:
            return self.running[0]
        except IndexError:
            return None

    @staticmethod
    def parse(json):

        return Experiments(
            [Experiment.parse(e) for e in ((json and json.get("draft") or []))],
            [Experiment.parse(e) for e in ((json and json.get("running") or []))]
        )


class Experiment(object):
    """
    Represents an experiment

    :ivar str id:
    :ivar str google_id:
    :ivar str name:
    :ivar array<Variation> variations:
    """

    def __init__(self, eid, google_id, name, variations):
        self.id = eid
        self.google_id = google_id
        self.name = name
        self.variations = variations

    @staticmethod
    def parse(json):
        return Experiment(
            json.get("id"),
            json.get("googleId"),
            json.get("name"),
            [Variation.parse(v) for v in json.get("variations")]
        )


class Variation(object):
    """
    Represents a variation of an experiment

    :ivar str id:
    :ivar str ref:
    :ivar str label:
    """

    def __init__(self, vid, ref, label):
        self.id = vid
        self.ref = ref
        self.label = label

    @staticmethod
    def parse(json):
        return Variation(
            json.get("id"),
            json.get("ref"),
            json.get("label")
        )




"""
Prismic.io python library.
"""

__title__ = 'prismic'
__version__ = '1.0.2'
__author__ = 'Prismic Team'
__license__ = 'Apache 2'

from .api import get, Api, SearchForm, Document
from .fragments import Fragment

# Set a default logger to prevent "No handler found" warnings
import logging
try:  # Python >=2.7
    from logging import NullHandler
except ImportError:
    class NullHandler(logging.Handler):
        def emit(self, record):
            pass

logging.getLogger(__name__).addHandler(NullHandler())

EXPERIMENTS_COOKIE = "io.prismic.experiment"
PREVIEW_COOKIE = "io.prismic.preview"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Experiments Tests for Prismic library"""

from __future__ import (absolute_import, division, print_function, unicode_literals)

import json
import prismic
import unittest
from prismic.experiments import Experiments

experiments_json = """
{
    "draft": [
        {
            "id": "xxxxxxxxxxoGelsX",
            "name": "Exp 2",
            "variations": [
                {
                    "id": "VDUBBawGAKoGelsZ",
                    "label": "Base",
                    "ref": "VDUBBawGALAGelsa"
                },
                {
                    "id": "VDUE-awGALAGemME",
                    "label": "var 1",
                    "ref": "VDUUmHIKAZQKk9uq"
                }
            ]
        }
    ],
    "running": [
        {
            "googleId": "_UQtin7EQAOH5M34RQq6Dg",
            "id": "VDUBBawGAKoGelsX",
            "name": "Exp 1",
            "variations": [
                {
                    "id": "VDUBBawGAKoGelsZ",
                    "label": "Base",
                    "ref": "VDUBBawGALAGelsa"
                },
                {
                    "id": "VDUE-awGALAGemME",
                    "label": "var 1",
                    "ref": "VDUUmHIKAZQKk9uq"
                }
            ]
        }
    ]
}"""


class ExperimentsTestCase(unittest.TestCase):

    def setUp(self):
        self.experiments = Experiments.parse(json.loads(experiments_json))

    def test_parsing(self):
        first = self.experiments.current()
        self.assertEqual(first.id, 'VDUBBawGAKoGelsX')
        self.assertEqual(first.google_id, '_UQtin7EQAOH5M34RQq6Dg')
        self.assertEqual(first.name, 'Exp 1')

    def test_cookie_parsing(self):
        self.assertIsNone(self.experiments.ref_from_cookie(''), 'Empty cookie')
        self.assertIsNone(self.experiments.ref_from_cookie('Ponies are awesome'), 'Invalid content')

        self.assertEqual('VDUBBawGALAGelsa',
                         self.experiments.ref_from_cookie('_UQtin7EQAOH5M34RQq6Dg%200'),
                         'Actual running variation')
        self.assertEqual('VDUUmHIKAZQKk9uq',
                         self.experiments.ref_from_cookie('_UQtin7EQAOH5M34RQq6Dg%201'),
                         'Actual running variation')

        self.assertIsNone(self.experiments.ref_from_cookie('_UQtin7EQAOH5M34RQq6Dg%209'), 'Index overflow')
        self.assertIsNone(self.experiments.ref_from_cookie('_UQtin7EQAOH5M34RQq6Dg%20-1'), 'Negative index overflow')
        self.assertIsNone(self.experiments.ref_from_cookie('NotAGoodLookingId%200'), 'Unknown Google ID')
        self.assertIsNone(self.experiments.ref_from_cookie('NotAGoodLookingId%201'), 'Unknown Google ID')



# -*- coding: utf-8 -*-

"""
prismic.predicates
~~~~~~~~~~~

This module provides helpers to write predicates to query Prismic

"""
import datetime


def at(fragment, value):
    return ['at', fragment, value]


def any(fragment, values):
    return ['any', fragment, values]


def fulltext(fragment, values):
    return ['fulltext', fragment, values]


def similar(fragment, value):
    return ['similar', fragment, value]


def gt(fragment, value):
    return ['number.gt', fragment, value]


def lt(fragment, value):
    return ['number.lt', fragment, value]


def in_range(fragment, before, after):
    return ['number.inRange', fragment, before, after]


def date_before(fragment, before):
    return ['date.before', fragment, __to_timestamp(before)]


def date_after(fragment, after):
    return ['date.after', fragment, __to_timestamp(after)]


def date_between(fragment, before, after):
    return ['date.between', fragment, __to_timestamp(before), __to_timestamp(after)]


def day_of_month(fragment, day):
    return ['date.day-of-month', fragment, day]


def day_of_month_after(fragment, day):
    return ['date.day-of-month-after', fragment, day]


def day_of_month_before(fragment, day):
    return ['date.day-of-month-before', fragment, day]


def day_of_week(fragment, day):
    return ['date.day-of-week', fragment, day]


def day_of_week_after(fragment, day):
    return ['date.day-of-week-after', fragment, day]


def day_of_week_before(fragment, day):
    return ['date.day-of-week-before', fragment, day]


def month(fragment, month):
    return ['date.month', fragment, month]


def month_before(fragment, month):
    return ['date.month-before', fragment, month]


def month_after(fragment, month):
    return ['date.month-after', fragment, month]


def year(fragment, year):
    return ['date.year', fragment, year]


def hour(fragment, hour):
    return ['date.hour', fragment, hour]


def hour_before(fragment, hour):
    return ['date.hour-before', fragment, hour]


def hour_after(fragment, hour):
    return ['date.hour-after', fragment, hour]


def near(fragment, latitude, longitude, radius):
    return ['geopoint.near', fragment, latitude, longitude, radius]


def __to_timestamp(input):
    if isinstance(input, datetime.date) or isinstance(input, datetime.datetime):
        return (input - datetime.datetime(1970, 1, 1)).total_seconds() * 1000
    return input



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for Prismic library"""

from __future__ import (absolute_import, division, print_function, unicode_literals)

from prismic.cache import ShelveCache, NoCache
from prismic.exceptions import InvalidTokenError, AuthorizationNeededError, \
    UnexpectedError
from .test_prismic_fixtures import fixture_api, fixture_search, fixture_groups, \
    fixture_structured_lists, fixture_empty_paragraph, fixture_store_geopoint, \
    fixture_image_links, fixture_spans_labels, fixture_block_labels, fixture_custom_html
import json
import logging
import datetime
import prismic
from prismic import predicates
import unittest

# logging.basicConfig(level=logging.DEBUG)
# log = logging.getLogger(__name__)


class DocTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        """Teardown."""

    def test_api(self):
        # startgist:13972e11d41ca29c0171:prismic-api.py
        api = prismic.get("https://lesbonneschoses.cdn.prismic.io/api")
        # endgist
        self.assertIsNotNone(api)

    def test_simplequery(self):
        # startgist:848271920fc502fb82d1:prismic-simplequery.py
        api = prismic.get("https://lesbonneschoses.prismic.io/api")
        response = api.form("everything").ref(api.get_master())\
            .query(predicates.at("document.type", "product"))\
            .submit()
        # endgist
        self.assertEqual(response.results_size, 16)

    def test_api_private(self):
        try:
            # startgist:71fbe142e1078f0ff76b:prismic-apiPrivate.py
            # This will fail because the token is invalid, but this is how to access a private API
            api = prismic.get('https://lesbonneschoses.prismic.io/api', 'MC5-XXXXXXX-vRfvv70')
            # endgist
            self.fail('Should have thrown')  # gisthide
        except InvalidTokenError as e:
            pass

    def test_references(self):
        # startgist:8061f7a1028d4f55dd63:prismic-references.py
        preview_token = 'MC5VbDdXQmtuTTB6Z0hNWHF3.c--_vVbvv73vv73vv73vv71EA--_vS_vv73vv70T77-9Ke-_ve-_vWfvv70ebO-_ve-_ve-_vQN377-9ce-_vRfvv70'
        api = prismic.get('https://lesbonneschoses.prismic.io/api', preview_token)
        st_patrick_ref = api.get_ref('St-Patrick specials')
        # Now we'll use this reference for all our calls
        response = api.form("everything")\
            .ref(st_patrick_ref)\
            .query(predicates.at("document.type", "product"))\
            .submit()
        # The documents object contains a Response object with all documents of type "product"
        # including the new "Saint-Patrick's Cupcake"
        # endgist
        self.assertEqual(response.results_size, 17)

    def test_orderings(self):
        # startgist:88098ee61f2f982ae9d6:prismic-orderings.py
        api = prismic.get('https://lesbonneschoses.prismic.io/api')
        response = api.form('everything')\
            .ref(api.get_master())\
            .query(predicates.at("document.type", "product"))\
            .pageSize(100)\
            .orderings('[my.product.price desc]')\
            .submit()
        # The products are now ordered by price, highest first
        results = response.results
        # endgist
        self.assertEqual(response.results_per_page, 100)

    def test_predicates(self):
        # startgist:75f170c0950a97b57896:prismic-predicates.py
        api = prismic.get("http://lesbonneschoses.prismic.io/api")
        response = api.form("everything").ref(api.get_master()) \
            .query(predicates.at("document.type", "product"),
                   predicates.date_after("my.blog-post.date", 1401580800000))\
            .submit()
        # endgist
        self.assertEqual(response.results_size, 0)

    def test_all_predicates(self):
        # startgist:057350d746fa64a673df:prismic-allPredicates.py
        # "at" predicate: equality of a fragment to a value.
        at = predicates.at("document.type", "article")
        # "any" predicate: equality of a fragment to a value.
        any = predicates.any("document.type", ["article", "blog-post"])
        # "fulltext" predicate: fulltext search in a fragment.
        fulltext = predicates.fulltext("my.article.body", "sausage")
        # "similar" predicate, with a document id as reference
        similar = predicates.similar("UXasdFwe42D", 10)
        # endgist
        self.assertEqual(at, ["at", "document.type", "article"])
        self.assertEqual(any, ["any", "document.type", ["article", "blog-post"]])

    def test_as_html(self):
        api = prismic.get("http://lesbonneschoses.prismic.io/api")
        response = api.form("everything").ref(api.get_master())\
            .query(predicates.at("document.id", "UlfoxUnM0wkXYXbX")).submit()
        # startgist:12326c99257882585e0d:prismic-asHtml.py
        def link_resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)
        doc = response.documents[0]
        html = doc.as_html(link_resolver)
        # endgist
        self.assertIsNotNone(html)

    def test_html_serializer(self):
        # startgist:dcf171c8e047f43e0472:prismic-htmlSerializer.py
        api = prismic.get("http://lesbonneschoses.prismic.io/api")
        response = api.form("everything").ref(api.get_master()) \
            .query(predicates.at("document.id", "UlfoxUnM0wkXYXbX")).submit()

        def link_resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)

        def html_serializer(element, content):
            if isinstance(element, prismic.fragments.Block.Image):
                # Don't wrap images in a <p> tag
                return element.get_view().as_html(link_resolver)
            if isinstance(element, prismic.fragments.Span.Hyperlink):
                # Add a class to links
                return """<a class="some-link" href="%s">""" % element.get_url(link_resolver) + content + "</a>"
            return None

        doc = response.documents[0]
        html = doc.get_structured_text("blog-post.body").as_html(link_resolver, html_serializer)
        # endgist
        self.assertIsNotNone(html)

    def test_get_text(self):
        api = prismic.get('https://lesbonneschoses.prismic.io/api')
        response = api.form('everything').query(predicates.at("document.id", "UlfoxUnM0wkXYXbl"))\
            .ref(api.get_master()).submit()
        doc = response.documents[0]
        # // startgist:d06042ce84f6002c055d:prismic-getText.py
        author = doc.get_text("blog-post.author")
        if author is None:
            author = "Anonymous"
        # endgist
        self.assertEqual(author, "John M. Martelle, Fine Pastry Magazine")

    def test_get_number(self):
        api = prismic.get('https://lesbonneschoses.prismic.io/api')
        response = api.form('everything').query(predicates.at("document.id", "UlfoxUnM0wkXYXbO"))\
            .ref(api.get_master()).submit()
        doc = response.documents[0]
        # startgist:2542ea501c2f950c597e:prismic-getNumber.py
        # Number predicates
        gt = predicates.gt("my.product.price", 10)
        lt = predicates.lt("my.product.price", 20)
        in_range = predicates.in_range("my.product.price", 10, 20)

        # Accessing number fields
        price = doc.get_number("product.price").value
        # endgist
        self.assertEqual(price, 2.5)

    def test_images(self):
        api = prismic.get('https://lesbonneschoses.prismic.io/api')
        response = api.form('everything').query(predicates.at("document.id", "UlfoxUnM0wkXYXbO")) \
            .ref(api.get_master()).submit()
        doc = response.documents[0]
        # startgist:360d054f79680bd48679:prismic-images.py
        # Accessing image fields
        image = doc.get_image('product.image')
        # By default the 'main' view is returned
        url = image.url
        # endgist
        self.assertEqual(
            url,
            'https://prismic-io.s3.amazonaws.com/lesbonneschoses/f606ad513fcc2a73b909817119b84d6fd0d61a6d.png')

    def test_date_timestamp(self):
        api = prismic.get('https://lesbonneschoses.prismic.io/api')
        response = api.form('everything').query(predicates.at("document.id", "UlfoxUnM0wkXYXbl"))\
            .ref(api.get_master()).submit()
        doc = response.documents[0]
        # startgist:a416bcf69d3e604d5c3b:prismic-dateTimestamp.py
        # Date and Timestamp predicates
        date_before = predicates.date_before("my.product.releaseDate", datetime.datetime(2014, 6, 1))
        date_after = predicates.date_after("my.product.releaseDate", datetime.datetime(2014, 1, 1))
        date_Between = predicates.date_between("my.product.releaseDate", datetime.datetime(2014, 1, 1), datetime.datetime(2014, 6, 1))
        day_of_month = predicates.day_of_month("my.product.releaseDate", 14)
        day_of_month_after = predicates.day_of_month_after("my.product.releaseDate", 14)
        day_of_month_before = predicates.day_of_month_before("my.product.releaseDate", 14)
        day_of_week = predicates.day_of_week("my.product.releaseDate", "Tuesday")
        day_of_week_after = predicates.day_of_week_after("my.product.releaseDate", "Wednesday")
        day_of_week_before = predicates.day_of_month_before("my.product.releaseDate", "Wednesday")
        month = predicates.month("my.product.releaseDate", "June")
        month_before = predicates.month_before("my.product.releaseDate", "June")
        month_after = predicates.month_after("my.product.releaseDate", "June")
        year = predicates.year("my.product.releaseDate", 2014)
        hour = predicates.hour("my.product.releaseDate", 12)
        hour_before = predicates.hour_before("my.product.releaseDate", 12)
        hour_after = predicates.hour_after("my.product.releaseDate", 12)

        # Accessing Date and Timestamp fields
        date = doc.get_date("blog-post.date")
        date_year = date and date.as_datetime.year
        update_time = doc.get_timestamp("blog-post.update")
        update_hour = update_time and update_time.as_datetime.hours
        # endgist
        self.assertEqual(date_year, 2013)

    def test_group(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"documents\":{\"type\":\"Group\",\"value\":[{\"linktodoc\":{\"type\":\"Link.document\",\"value\":{\"document\":{\"id\":\"UrDejAEAAFwMyrW9\",\"type\":\"doc\",\"tags\":[],\"slug\":\"installing-meta-micro\"},\"isBroken\":false}},\"desc\":{\"type\":\"StructuredText\",\"value\":[{\"type\":\"paragraph\",\"text\":\"A detailed step by step point of view on how installing happens.\",\"spans\":[]}]}},{\"linktodoc\":{\"type\":\"Link.document\",\"value\":{\"document\":{\"id\":\"UrDmKgEAALwMyrXA\",\"type\":\"doc\",\"tags\":[],\"slug\":\"using-meta-micro\"},\"isBroken\":false}}}]}}}}"
        document = prismic.Document(json.loads(data))
        def resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)
        # startgist:8618c84784a22634ca6a:prismic-group.py
        group = document.get_group("article.documents")
        docs = (group and group.value) or []
        for doc in docs:
            desc = doc.get_structured_text("desc")
            link = doc.get_link("linktodoc")
        # endgist
        self.assertEqual(docs[0].get_structured_text("desc").as_html(resolver), "<p>A detailed step by step point of view on how installing happens.</p>")

    def test_link(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"source\":{\"type\":\"Link.document\",\"value\":{\"document\":{\"id\":\"UlfoxUnM0wkXYXbE\",\"type\":\"product\",\"tags\":[\"Macaron\"],\"slug\":\"dark-chocolate-macaron\"},\"isBroken\":false}}}}}"
        document = prismic.Document(json.loads(data))
        # startgist:1417c3a9baf3015e34b8:prismic-link.py
        def resolver(document_link):
            return "/document/%s/%s" % (document_link.id, document_link.slug)
        source = document.get_link("article.source")
        url = source and source.get_url(resolver)
        # endgist
        self.assertEqual(url, "/document/UlfoxUnM0wkXYXbE/dark-chocolate-macaron")

    def test_embed(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"video\":{\"type\":\"Embed\",\"value\":{\"oembed\":{\"provider_url\":\"http://www.youtube.com/\",\"type\":\"video\",\"thumbnail_height\":360,\"height\":270,\"thumbnail_url\":\"http://i1.ytimg.com/vi/baGfM6dBzs8/hqdefault.jpg\",\"width\":480,\"provider_name\":\"YouTube\",\"html\":\"<iframe width=\\\"480\\\" height=\\\"270\\\" src=\\\"http://www.youtube.com/embed/baGfM6dBzs8?feature=oembed\\\" frameborder=\\\"0\\\" allowfullscreen></iframe>\",\"author_name\":\"Siobhan Wilson\",\"version\":\"1.0\",\"author_url\":\"http://www.youtube.com/user/siobhanwilsonsongs\",\"thumbnail_width\":480,\"title\":\"Siobhan Wilson - All Dressed Up\",\"embed_url\":\"https://www.youtube.com/watch?v=baGfM6dBzs8\"}}}}}}"
        document = prismic.Document(json.loads(data))
        # startgist:f405d95bc1d987aedeaa:prismic-embed.py
        video = document.get_embed("article.video")
        # Html is the code to include to embed the object, and depends on the embedded service
        html = video and video.as_html
        # endgist
        self.assertEqual(html, "<div data-oembed=\"https://www.youtube.com/watch?v=baGfM6dBzs8\" data-oembed-type=\"video\" data-oembed-provider=\"YouTube\"><iframe width=\"480\" height=\"270\" src=\"http://www.youtube.com/embed/baGfM6dBzs8?feature=oembed\" frameborder=\"0\" allowfullscreen></iframe></div>")

    def test_color(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"background\":{\"type\":\"Color\",\"value\":\"#000000\"}}}}"
        document = prismic.Document(json.loads(data))
        # startgist:85a630064368ebabf2e6:prismic-color.py
        bgcolor = document.get_color("article.background")
        hex = bgcolor.value
        # endgist
        self.assertEqual(hex, "#000000")

    def test_geopoint(self):
        data = "{\"id\":\"abcd\",\"type\":\"article\",\"href\":\"\",\"slugs\":[],\"tags\":[],\"data\":{\"article\":{\"location\":{\"type\":\"GeoPoint\",\"value\":{\"latitude\":48.877108,\"longitude\":2.333879}}}}}"
        document = prismic.Document(json.loads(data))
        # startgist:316cb72c1a5789fd2049:prismic-geopoint.py
        # "near" predicate for GeoPoint fragments
        near = predicates.near("my.store.location", 48.8768767, 2.3338802, 10)

        # Accessing GeoPoint fragments
        place = document.get_geopoint("article.location")
        coordinates = place and ("%.6f,%.6f" % (place.latitude, place.longitude))
        # endgist
        self.assertEqual(coordinates, "48.877108,2.333879")

    def test_cache(self):
        # startgist:11a75574ad42c60ff6de:prismic-cache.py
        # Just implement your own cache object by duck-typing
        # https://github.com/prismicio/python-kit/blob/master/prismic/cache.py
        no_cache = NoCache()
        # This api will use the custom cache object
        api = prismic.get('https://lesbonneschoses.prismic.io/api', cache=no_cache)
        # endgist
        self.assertIsNotNone(api)


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys

from setuptools import setup, find_packages
import setuptools

setup(
    name='prismic',
    version='1.0.13',
    description='Prismic.io development kit',
    author='The Prismic.io Team',
    author_email='contact@prismic.io',
    url='http://prismic.io',
    license='Apache 2',
    packages=find_packages(),
    test_suite='tests',
    classifiers=[
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2.7'
    ],
    install_requires=[
        'requests'
    ]
)



# -*- coding: utf-8 -*-

"""
prismic.api
~~~~~~~~~~~

This module implements the Prismic API.

"""

from __future__ import (absolute_import, division, print_function, unicode_literals)

import sys
import platform
import pkg_resources
from copy import copy, deepcopy
from collections import OrderedDict
from requests.exceptions import InvalidSchema
from prismic.experiments import Experiments
from prismic import predicates

try:  # 2.7
    import urllib.parse as urlparse
except ImportError:  # 3.x
    import urllib as urlparse
import requests
import json
import re

from .exceptions import (InvalidTokenError, AuthorizationNeededError,
                         HTTPError, InvalidURLError, RefMissing)
from .fragments import Fragment
from .cache import ShelveCache
from .utils import string_types
import logging

log = logging.getLogger(__name__)


def get(url, access_token=None, cache=None):
    """Fetches the prismic api JSON.
    Returns :class:`Api <Api>` object.

    :param url: URL to the api of the repository (mandatory).
    :param access_token: The access token (optional).
    :param cache: The cache object. Optional, will default to a file-based cache if None is passed.
    """
    return Api(_get_json(url, access_token=access_token, cache=cache, ttl=5), access_token, cache)


def _get_json(url, params=None, access_token=None, cache=None, ttl=None):
    full_params = dict() if params is None else params.copy()
    if cache is None:
        cache = ShelveCache(re.sub(r'/\\', '', url.split('/')[2]))
    if access_token is not None:
        full_params["access_token"] = access_token
    full_url = url if len(full_params) == 0 else (url + "?" + urlparse.urlencode(full_params, doseq=1))
    cached = cache.get(full_url)
    if cached is not None:
        return cached
    try:
        r = requests.get(full_url, headers={
            "Accept": "application/json",
            "User-Agent": "Prismic-python-kit/%s Python/%s" % (pkg_resources.require("prismic")[0].version, platform.python_version())
        })
        if r.status_code == 200:
            text_result = r.text
            if not isinstance(text_result, str):
                text_result = text_result.decode('utf-8')
            json_result = json.loads(text_result, object_pairs_hook=OrderedDict)
            expire = ttl or _max_age(r)
            if expire is not None:
                cache.set(full_url, json_result, expire)
            return json_result
        elif r.status_code == 401:
            if len(access_token) == 0:
                raise AuthorizationNeededError()
            else:
                raise InvalidTokenError()
        else:
            raise HTTPError(r.status_code, str(r.text))
    except InvalidSchema as e:
        raise InvalidURLError(e)


def _max_age(response):
    expire_header = response.headers["Cache-Control"]
    if expire_header is not None:
        m = re.match("max-age=(\d+)", expire_header)
        if m:
            return int(m.group(1))
    return None


class Api(object):
    """
    A Prismic API, pointing to a specific repository. Use prismic.api.get() to fetch one.

    :ivar dict bookmarks: all bookmarks, as a dict from name to document id
    :ivar array<str> types: all available types
    :ivar array<str> tags: all available tags
    :ivar Experiments experiments: information about current experiments
    :ivar str access_token: current access token (may be None)
    """

    def __init__(self, data, access_token, cache):
        self.cache = cache
        self.refs = [Ref(ref) for ref in data.get("refs")]
        self.bookmarks = data.get("bookmarks")
        self.types = data.get("types")
        self.tags = data.get("tags")
        self.forms = data.get("forms")
        for name in self.forms:
            fields = self.forms[name].get("fields")
            for field in fields:
                if field == "q":
                    fields[field].update({"multiple": True})
        self.experiments = Experiments.parse(data.get("experiments"))
        self.oauth_initiate = data.get("oauth_initiate")
        self.oauth_token = data.get("oauth_token")
        self.access_token = access_token

        self.master = ([ref for ref in self.refs if ref.is_master_ref][:1] or [None])[0]
        if not self.master:
            log.error("No master reference found")

    def preview_session(self, token, link_resolver, default_url):
        """Return the URL to display a given preview

        :param token as received from Prismic server to identify the content to preview
        :param link_resolver the link resolver to build URL for your site
        :param default_url the URL to default to return if the preview doesn't correspond to a document
                       (usually the home page of your site)

        :return: the URL to redirect the user to
        """
        main_document_id = _get_json(token).get("mainDocument")
        if main_document_id is None:
            return default_url
        response = self.form("everything").ref(token).query(predicates.at("document.id", main_document_id)).submit()
        if len(response.results) == 0:
            return default_url
        return link_resolver(response.documents[0].as_link())

    def get_ref(self, label):
        """Get the :class:`Ref <Ref>` with a specific label.
        Returns :class:`Ref <Ref>` object.

        :param label: Name of the label.
        """
        ref = [ref for ref in self.refs if ref.label == label]
        return ref[0] if ref else None

    def get_master(self):
        """Returns current master :class:`Ref <Ref>` object."""
        return self.master

    def form(self, name):
        """Constructs the form with data from Api.
        Returns :class:`SearchForm <SearchForm>` object.

        :param name: Name of the form.
        """
        form = self.forms.get(name)
        if form is None:
            raise Exception("Bad form name %s, valid form names are: %s" % (name, ', '.join(self.forms)))
        return SearchForm(self.forms.get(name), self.access_token, self.cache)


class Ref(object):
    """
    A Prismic.io Reference (corresponds to a release)
    """

    def __init__(self, data):
        self.id = data.get("id")
        self.ref = data.get("ref")
        self.label = data.get("label")
        self.is_master_ref = data.get("isMasterRef")
        self.scheduled_at = data.get("scheduledAt")


class SearchForm(object):
    """Form to search for documents. Most of the methods return self object to allow chaining.
    """

    def __init__(self, form, access_token, cache):
        self.action = form.get("action")
        self.method = form.get("method")
        self.enctype = form.get("enctype")
        self.fields = form.get("fields") or {}
        self.data = {}
        # default values
        for field, value in list(self.fields.items()):
            if value.get("default"):
                self.set(field, value["default"])
        self.access_token = access_token
        self.cache = cache

    def ref(self, ref):
        """:param ref: A :class:`Ref <Ref>` object or an string."""

        if isinstance(ref, Ref):
            ref = ref.ref

        return self.set('ref', ref)

    @staticmethod
    def _serialize(field):
        if isinstance(field, string_types):
            if field.startswith('my.') or field.startswith('document.'):
                return field
            else:
                return '"' + field + '"'
        elif hasattr(field, '__iter__'):
            strings = []
            for item in field:
                strings.append(SearchForm._serialize(item))
            return "[" + ", ".join(strings) + "]"
        else:
            return str(field)

    def query(self, *argv):
        """:param argv: Either a string query, or any number of Array corresponding to predicates.

        See the :mod:`prismic.predicates <prismic.predicate>` module for helper functions.
        """
        if len(argv) == 0:
            return self
        if isinstance(argv[0], string_types):
            q = argv[0]
        else:
            q = "["
            for predicate in argv:
                op = predicate[0]
                args = []
                for arg in predicate[1:]:
                    args.append(SearchForm._serialize(arg))
                q += "[:d = %(op)s(%(args)s)]" % {
                    'op': op,
                    'args': ", ".join(args)
                }
            q += "]"
        return self.set('q', q)

    def set(self, field, value):
        form_field = self.fields.get(field)
        if form_field and form_field.get("multiple"):
            if not self.data.get(field):
                self.data.update({field: []})
            self.data[field].append(value)
        else:
            self.data.update({field: value})
        return self

    def orderings(self, orderings):
        """Sets the query orderings

        :param orderings String with the orderings predicate
        :returns: the SearchForm instance to chain calls
        """
        return self.set("orderings", orderings)

    def submit_assert_preconditions(self):
        if self.data.get('ref') is None:
            raise RefMissing()

    def submit(self):
        """
        Submit the query to the Prismic.io server

        :return: :class:`Response <prismic.api.Response>`
        """
        self.submit_assert_preconditions()
        return Response(_get_json(self.action, self.data, self.access_token, self.cache))

    def page(self, page_number):
        """Set query page number

        :param page_number: int representing the page number
        """
        return self.set("page", page_number)

    def page_size(self, nb_results):
        """Set query page size

        :param nb_results: int representing the number of results per page
        """
        return self.set("pageSize", nb_results)

    def pageSize(self, nb_results):
        """Deprecated: use page_size instead
        """
        return self.page_size(nb_results)

    def count(self):
        """Count the total number of results
        """
        return copy(self).pageSize(1).submit().total_results_size

    def __copy__(self):
        cp = type(self)({}, self.access_token, self.cache)
        cp.action = deepcopy(self.action)
        cp.method = deepcopy(self.method)
        cp.enctype = deepcopy(self.enctype)
        cp.fields = deepcopy(self.fields)
        cp.data = deepcopy(self.data)
        return cp


class Response(object):
    """
    Prismic's response to a query.

    :ivar array<prismic.api.Document> documents: the documents of the current page
    :ivar int page: the page in this result, starting by 1
    :ivar int results_per_page: max result in a page
    :ivar int total_results_size: total number of results for this query
    :ivar int total_pages: total number of pages for this query
    :ivar str next_page: URL of the next page (may be None if on the last page )
    :ivar str prev_page: URL of the previous page (may be None)
    :ivar int results_size: number of results actually returned for the current page
    """

    def __init__(self, data):
        self._data = data
        self.documents = [Document(d) for d in data.get("results")]
        self.page = data.get("page")
        self.next_page = data.get("next_page")
        self.prev_page = data.get("prev_page")
        self.results_per_page = data.get("results_per_page")
        self.total_pages = data.get("total_pages")
        self.total_results_size = data.get("total_results_size")
        self.results_size = data.get("results_size")

    def __getattr__(self, name):
        return self._data.get(name)

    def __repr__(self):
        return "Response %s" % self._data


class LinkedDocument(object):
    """
    Represents a link to a document
    """

    def __init__(self, data):
        self._data = data

    def get_id(self):
        return self._data.get("id")

    def get_slug(self):
        return self._data.get("slug")

    def get_typ(self):
        return self._data.get("typ")

    def get_tags(self):
        return self._data.get("tags")


class Document(Fragment.WithFragments):
    """
    Represents a Prismic.io Document

    :ivar str id: document id
    :ivar str type:
    :ivar str href:
    :ivar array<str> tags:
    :ivar array<str> slugs:
    :ivar array<LinkedDocuments> linked_documents:
    """

    def __init__(self, data):
        Fragment.WithFragments.__init__(self, {})
        self._data = data
        self.linked_documents = []

        fragments = data.get("data").get(self.type) if "data" in data else {}
        for (fragment_name, fragment_value) in list(fragments.items()):
            f_key = "%s.%s" % (self.type, fragment_name)

            if isinstance(fragment_value, list):
                for index, fragment_value_element in enumerate(fragment_value):
                    self.fragments["%s[%s]" % (f_key, index)] = Fragment.from_json(
                        fragment_value_element)

            elif isinstance(fragment_value, dict):
                self.fragments[f_key] = Fragment.from_json(fragment_value)

        if "linked_documents" in data:
            self.linked_documents = [LinkedDocument(d) for d in data.get("linked_documents")]

        self.slugs = ["-"]
        if data.get("slugs") is not None:
            self.slugs = [Document.__unquote(slug) for slug in data.get("slugs")]

    @staticmethod
    def __unquote(s):
        if sys.version_info >= (3, 0):
            return urlparse.unquote(s)
        else:
            return urlparse.unquote(s.encode('utf8')).decode('utf8')

    def as_link(self):
        """
        Convert the current document to a DocumentLink

        :return: :class:`DocumentLink <prismic.api.Fragment.DocumentLink>`
        """
        data = self._data.copy()
        data['slug'] = self.slug
        return Fragment.DocumentLink({
            'document': data
        })

    def __getattr__(self, name):
        return self._data.get(name)

    @property
    def slug(self):
        """
        Return the most recent slug

        :return: str slug
        """
        return self.slugs[0] if self.slugs else "-"

    def __repr__(self):
        return "Document %s" % self.fragments
